@@ -1,14 +1,19 @@
 # =============================================================================
 # Minet CrowdTangle API Client
-# =============================================================================
+# ================================s=============================================
 #
 # A unified CrowdTangle API client that can be used to keep an eye on the
 # rate limit and the used token etc.
 #
 import json
 
-from minet.utils import RateLimiterState, rate_limited_method
-from minet.web import create_pool, create_request_retryer, retrying_method, request
+from minet.rate_limiting import ThreadsafeBurstyRateLimiterState, rate_limited_method
+from minet.web import (
+    create_pool_manager,
+    threadsafe_retrying_method,
+    ThreadsafeRequestRetryers,
+    request,
+)
 from minet.crowdtangle.constants import (
     CROWDTANGLE_DEFAULT_TIMEOUT,
     CROWDTANGLE_DEFAULT_RATE_LIMIT,
@@ -20,6 +25,7 @@ from minet.crowdtangle.exceptions import (
     CrowdTangleRateLimitExceeded,
     CrowdTangleInvalidRequestError,
     CrowdTangleServerError,
+    CrowdTanglePostNotFound,
 )
 from minet.crowdtangle.leaderboard import crowdtangle_leaderboard
 from minet.crowdtangle.lists import crowdtangle_lists
@@ -30,7 +36,7 @@ from minet.crowdtangle.summary import crowdtangle_summary
 
 
 class CrowdTangleAPIClient(object):
-    def __init__(self, token, rate_limit=None, before_sleep=None):
+    def __init__(self, token, rate_limit=None):
         if rate_limit is None:
             rate_limit = CROWDTANGLE_DEFAULT_RATE_LIMIT
             summary_rate_limit = CROWDTANGLE_LINKS_DEFAULT_RATE_LIMIT
@@ -39,23 +45,22 @@ class CrowdTangleAPIClient(object):
             summary_rate_limit = rate_limit
 
         self.token = token
-        self.rate_limiter_state = RateLimiterState(rate_limit, period=60)
-        self.summary_rate_limiter_state = RateLimiterState(
+        self.rate_limiter_state = ThreadsafeBurstyRateLimiterState(
+            rate_limit, period=60
+        )
+        self.summary_rate_limiter_state = ThreadsafeBurstyRateLimiterState(
             summary_rate_limit, period=60
         )
-        self.pool = create_pool(timeout=CROWDTANGLE_DEFAULT_TIMEOUT)
-        self.retryer = create_request_retryer(
-            additional_exceptions=[CrowdTangleInvalidJSONError, CrowdTangleServerError],
-            before_sleep=before_sleep,
+        self.pool_manager = create_pool_manager(
+            timeout=CROWDTANGLE_DEFAULT_TIMEOUT, num_pools=1, parallelism=100
+        )
+        self.retryers = ThreadsafeRequestRetryers(
+            additional_exceptions=[CrowdTangleInvalidJSONError, CrowdTangleServerError]
         )
 
-    @retrying_method()
+    @threadsafe_retrying_method()
     def __request(self, url):
-        err, response = request(url, pool=self.pool)
-
-        # Debug
-        if err:
-            raise err
+        response = request(url, pool_manager=self.pool_manager)
 
         # Bad auth
         if response.status == 401:
@@ -71,13 +76,11 @@ class CrowdTangleAPIClient(object):
 
         # Bad params
         if response.status >= 400:
-            data = response.data.decode("utf-8")
-
             try:
-                data = json.loads(data)
+                data = response.json()
             except json.decoder.JSONDecodeError:
                 raise CrowdTangleInvalidRequestError(
-                    data, url=url, status=response.status
+                    response.text(), url=url, status=response.status
                 )
 
             raise CrowdTangleInvalidRequestError(
@@ -85,11 +88,14 @@ class CrowdTangleAPIClient(object):
             )
 
         try:
-            data = json.loads(response.data)["result"]
-        except (json.decoder.JSONDecodeError, TypeError, KeyError):
+            data = response.json()
+        except json.decoder.JSONDecodeError:
             raise CrowdTangleInvalidJSONError
 
-        return data
+        try:
+            return data["result"]
+        except KeyError:
+            raise CrowdTanglePostNotFound(data=data, url=url)
 
     @rate_limited_method("rate_limiter_state")
     def request(self, url):
