@@ -6,8 +6,6 @@
 #
 from datetime import date, datetime, timedelta
 
-from minet.crowdtangle.exceptions import CrowdTangleMissingStartDateError
-
 DAY_DELTA = timedelta(days=1)
 
 
@@ -105,9 +103,8 @@ def make_paginated_iterator(
         per_call=False,
         detailed=False,
         namespace=None,
-        **kwargs
+        **kwargs,
     ):
-
         if namespace is not None:
             kwargs = vars(namespace)
         else:
@@ -115,9 +112,6 @@ def make_paginated_iterator(
 
         # Checking we have the necessary dates
         if kwargs.get("sort_by") == "date":
-            if kwargs.get("start_date") is None:
-                raise CrowdTangleMissingStartDateError
-
             # Inferring end date to be now, this will be important later
             if kwargs.get("end_date") is None:
                 kwargs["end_date"] = infer_end_date()
@@ -169,7 +163,6 @@ def make_paginated_iterator(
 
             # We have exhausted the available data
             if items is None:
-
                 if need_to_chunk:
                     could_rotate = rotate_year()
 
@@ -185,7 +178,6 @@ def make_paginated_iterator(
             acc = []
 
             for item in items:
-
                 # Avoiding duplicating items due to race conditions
                 if item_id_getter(item) in last_items:
                     continue
