@@ -6,10 +6,10 @@
 #
 from urllib.parse import quote_plus
 
-from minet.web import request_json
+from minet.web import request
 from minet.mediacloud.constants import MEDIACLOUD_API_BASE_URL, MEDIACLOUD_DEFAULT_BATCH
 from minet.mediacloud.exceptions import MediacloudServerError
-from minet.mediacloud.formatters import format_story
+from minet.mediacloud.types import MediacloudStory
 from minet.mediacloud.utils import get_last_processed_stories_id
 
 
@@ -42,7 +42,6 @@ def query_additions(
     publish_month=None,
     publish_year=None,
 ):
-
     if collections is not None:
         query += create_plural_query_component("tags_id_media", collections)
 
@@ -75,7 +74,6 @@ def url_forge(
     count=False,
     last_processed_stories_id=None,
 ):
-
     url = "%s/stories_public/%s?key=%s" % (
         MEDIACLOUD_API_BASE_URL,
         "count" if count else "list",
@@ -106,7 +104,7 @@ def url_forge(
 
 
 def mediacloud_search(
-    pool,
+    pool_manager,
     token,
     query,
     filter_query=None,
@@ -135,10 +133,8 @@ def mediacloud_search(
                 last_processed_stories_id=last_processed_stories_id,
             )
 
-            err, response, data = request_json(url, pool=pool)
-
-            if err:
-                raise err
+            response = request(url, pool_manager=pool_manager)
+            data = response.json()
 
             if response.status >= 500:
                 raise MediacloudServerError(server_error=data.get("error"))
@@ -149,7 +145,7 @@ def mediacloud_search(
 
             for story in data:
                 if not raw:
-                    story = format_story(story)
+                    story = MediacloudStory.from_payload(story)
 
                 yield story
 
